#!/usr/bin/env python
"""
Find the closest points to a target location using the trace header database.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segysqldist.py,v 1.3 2005/06/22 17:50:48 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.3 $' # See man ident
__date__ = '$Date: 2005/06/22 17:50:48 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """Lots of people for helping me with my thesis.
Especially Neal Driscoll and Lisa Tauxe!
"""

import mmap
import os
import struct	# Convert two and from binary formats
import sqlite
import sys

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.3 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution


from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-f','--file',dest='dbfilename',default='segy.db',
		    help='file for the sqlite database  [default: %default]')
myparser.add_option('-x','--x',dest='x', help='x coordinate (decimal degrees longitude)')
myparser.add_option('-y','--y',dest='y', help='y coordinate (decimal degrees latitude)')

from closestN import ClosestPoints
    
if __name__ == "__main__":
    (options,args) = myparser.parse_args()
    #print options
    if not options.x or not options.y:
        print "You must specify both x and y\n"
        myparser.print_help()
        sys.exit("Goodbye and good ridance")

    dbName = options.dbfilename
    cx = sqlite.connect (dbName)
    cu = cx.cursor()
    print "Starting SQL select.  This make take quite a while"
    cu.execute('SELECT X,Y,traceNumber,fileKey,Shotpoint FROM segyTrace')
    print "Finished SQL select"
    target = (options.x,options.y)

    o = open ("point","w")
    o.write(options.x+" "+options.y+"\n")

    ########################################
    # N points
    ########################################
    if True:
        print "# Beginning search"
        cPts = ClosestPoints(target,10000)
        i = 0
        o = open ('data','w')
        print "Inspecting data:"
        for row in cu.fetchall():
            i += 1
            if 0== i % 1000: print i
            x = float(row['X'])/3600.
            y = float(row['Y'])/3600.
            o.write(str(x)+' '+str(y)+'\n')
            point = (float(x),float(y))
            data = (row['fileKey'],row['Shotpoint'],row['traceNumber'])
            cPts.checkPoint(point,data)
        o = open ('results','w')

        for p in cPts.points:
            s = ''
            for value in p:
                s += ("  %.6f" % value)
            s += '\n'
            o.write(s)


    ########################################
    # Single point - Much faster
    ########################################
    if False:
        bestFile = None
        bestShot = None
        bestX = None
        bestY = None
        bestDistance = 99999999999.

        for row in cu.fetchall():
            x = float(row['X'])/3600.
            y = float(row['Y'])/3600.
            dist = greatCircleDistKm((x,y),target)
            if dist < bestDistance:
                print x,y,row['traceNumber'], row['fileKey'],row['Shotpoint'],target,dist
                bestDistance = dist
                bestShot = row['Shotpoint']
                bestX = x
                bestY = y
                bestFile = row['fileKey']
        print "BEST: ",bestFile,bestShot,bestX,bestY,bestDistance
    
